@@ -4,11 +4,45 @@
 .redisEnv <- new.env()
 .redisEnv$current <- .redisEnv
 
-.redis <- function() 
+.redis <- function(e)
 {
-  if(!exists('con',envir=.redisEnv$current))
+  if(missing(e)) e = .redisEnv$current
+  if(!exists('con',envir=e))
     stop('Not connected, try using redisConnect()')
-  .redisEnv$current$con
+  e$con
+}
+
+.openConnection <- function(host, port, nodelay=FALSE, timeout=2678399L, envir=rredis:::.redisEnv$current)
+{
+  stopifnot(typeof(host)=="character")
+  stopifnot(class(port)=="numeric")
+  stopifnot(typeof(nodelay)=="logical")
+# We track the file descriptor of the new connection in a sneaky way
+  fds <- rownames(showConnections(all=TRUE))
+  con <- socketConnection(host, port, open="a+b",
+                          blocking=TRUE, timeout=timeout)
+  fd <- rownames(showConnections(all=TRUE))
+  fd <- as.integer(setdiff(fd,fds))
+  if(nodelay)
+  {
+    Nagle <- .Call("SOCK_NAGLE",fd,1L,PACKAGE="rredis")
+    if(Nagle!=1) warning("Unable to set nodelay.")
+  }
+# Stash state in the redis enivronment describing this connection:
+  assign('fd',fd,envir=envir)
+  assign('con',con,envir=envir)
+  assign('host',host,envir=envir)
+  assign('port',port,envir=envir)
+  assign('nodelay',nodelay,envir=envir)
+# Count is for pipelined communication, it keeps track of the number of
+# getResponse calls that are pending.
+  assign('count',0,envir=envir)
+  con
+}
+
+.closeConnection <- function(s)
+{
+  close(s)
 }
 
 # .redisError may be called by any function when a serious error occurs.
@@ -18,10 +52,10 @@
 {
   env <- .redisEnv$current
   con <- .redis()
-  close(con)
+  .closeConnection(con)
 # May stop with an error here on connect fail
-  con <- socketConnection(env$host, env$port,open='a+b', blocking=TRUE, timeout=env$timeout)
-  assign('con',con,envir=env)
+  con <- .openConnection(host=env$host,
+                         port=env$port, nodelay=env$nodelay, envir=env)
   if(!is.null(e)) print(as.character(e))
   stop(msg)
 }
@@ -47,95 +81,14 @@
   .redisError("Interrupted communincation with Redis",e)
 }
 
-.getResponse <- function(raw=FALSE)
-{
-  env <- .redisEnv$current
-tryCatch({
-  con <- .redis()
-#  socketSelect(list(con), timeout=10L)
-  l <- readLines(con=con, n=1)
-  if(length(l)==0) .burn("Empty")
-  tryCatch(
-    env$count <- max(env$count - 1,0),
-    error = function(e) assign('count', 0, envir=env)
-  )
-  s <- substr(l, 1, 1)
-  if (nchar(l) < 2) {
-    if(s == '+') {
-      # '+' is a valid retrun message on at least one cmd (RANDOMKEY)
-      return('')
-    }
-    .burn("Invalid")
-  }
-  switch(s,
-         '-' = stop(substr(l,2,nchar(l))),
-         '+' = substr(l,2,nchar(l)),
-         ':' = as.numeric(substr(l,2,nchar(l))),
-         '$' = {
-             n <- as.numeric(substr(l,2,nchar(l)))
-             if (n < 0) {
-               return(NULL)
-             }
-#             socketSelect(list(con),timeout=10L)
-             dat <- tryCatch(readBin(con, 'raw', n=n),
-                             error=function(e) .redisError(e$message))
-             m <- length(dat)
-             if(m==n) {
-#               socketSelect(list(con),timeout=10L)
-               l <- readLines(con,n=1)  # Trailing \r\n
-               if(raw)
-                 return(dat)
-               else
-                 return(tryCatch(unserialize(dat),
-                         error=function(e) rawToChar(dat)))
-             }
-# The message was not fully recieved in one pass.
-# We allocate a list to hold incremental messages and then concatenate it.
-# This perfromance enhancement was adapted from the Rbig server package, 
-# written by Steve Weston and Pat Shields.
-             rlen <- 50
-             j <- 1
-             r <- vector('list',rlen)
-             r[j] <- list(dat)
-             while(m<n) {
-# Short read; we need to retrieve the rest of this message.
-#               socketSelect(list(con),timeout=10L)
-               dat <- tryCatch(readBin(con, 'raw', n=(n-m)),
-                            error=function (e) .redisError(e$message))
-               j <- j + 1
-               if(j>rlen) {
-                 rlen <- 2*rlen
-                 length(r) <- rlen
-               }
-               r[j] <- list(dat)
-               m <- m + length(dat)
-             }
-#             socketSelect(list(con),timeout=10L)
-             l <- readLines(con,n=1)  # Trailing \r\n
-             length(r) <- j
-             if(raw)
-               do.call(c,r)
-             else
-               tryCatch(unserialize(do.call(c,r)),
-                      error=function(e) rawToChar(do.call(c,r)))
-           },
-         '*' = {
-           numVars <- as.integer(substr(l,2,nchar(l)))
-           if(numVars > 0L) {
-             replicate(numVars, .getResponse(raw=raw), simplify=FALSE)
-           } else NULL
-         },
-         stop('Unknown message type'))
-}, interrupt=function(e) .burn(e)
-)
-}
-
 #
 # .raw is just a shorthand wrapper for charToRaw:
 #
 .raw <- function(word) 
 {
-  tryCatch(charToRaw(word),warning=function(w) stop(w), error=function(e) stop(e))
+  tryCatch(charToRaw(word),
+           warning=function(w) stop(w),
+           error=function(e) stop(e))
 }
 
 # Expose the basic Redis interface to the user
@@ -146,7 +99,7 @@ redisCmd <- function(CMD, ..., raw=FALSE)
            if(is.character(x)) charToRaw(x)
            else(.cerealize(x))
        ))
-  if(raw) return(do.call('.redisRawCmd',a))
+  if(raw) a <- c(a,raw=TRUE)
   do.call('.redisCmd', a)
 }
 
@@ -168,36 +121,46 @@ redisCmd <- function(CMD, ..., raw=FALSE)
 {
   env <- .redisEnv$current
   con <- .redis()
+# Check to see if a rename list exists and use it if it does...we also
+# define a little helper function to handle replacing the command.
+# The rename list must have the form:
+# list(OLDCOMMAND="NEWCOMMAND", SOME_OTHER_CMD="SOME_OTHER_NEW_CMD",...)
+  rep <- c()
+  if(exists("rename",envir=.redisEnv)) rep = get("rename",envir=.redisEnv)
   f <- match.call()
+# Check for raw option (which means don't deserialize returned resuts)
+  raw <- FALSE
+  if(any("raw" %in% names(f)))
+  {
+    wr  <- which(names(f)=="raw")
+    raw <- f[[wr]]
+    f   <- f[-wr]
+  }
   n <- length(f) - 1
   hdr <- paste('*', as.character(n), '\r\n',sep='')
-#  socketSelect(list(con),timeout=10L, write=TRUE)
-#  cat(hdr, file=con)
   writeBin(.raw(hdr), con)
-tryCatch({
-  for(j in seq_len(n)) {
-    v <- eval(f[[j+1]],envir=sys.frame(-1))
-    if(!is.raw(v)) v <- .cerealize(v)
-    l <- length(v)
-    hdr <- paste('$', as.character(l), '\r\n', sep='')
-#    socketSelect(list(con),timeout=10L, write=TRUE)
-#    cat(hdr, file=con)
-    writeBin(.raw(hdr), con)
-#    socketSelect(list(con),timeout=10L, write=TRUE)
-    writeBin(v, con)
-#    socketSelect(list(con),timeout=10L, write=TRUE)
-#    cat('\r\n', file=con)
-    writeBin(.raw('\r\n'), con)
-  }
-},
-error=function(e) {.redisError("Invalid agrument");invisible()},
-interrupt=function(e) .burn(e)
-)
+  tryCatch({
+    for(j in seq_len(n)) {
+      if(j==1)
+        v <- .renameCommand(eval(f[[j+1]],envir=sys.frame(-1)), rep)
+      else
+        v <- eval(f[[j+1]],envir=sys.frame(-1))
+      if(!is.raw(v)) v <- .cerealize(v)
+      l <- length(v)
+      hdr <- paste('$', as.character(l), '\r\n', sep='')
+      writeBin(.raw(hdr), con)
+      writeBin(v, con)
+      writeBin(.raw('\r\n'), con)
+    }
+  },
+    error=function(e) {.redisError("Invalid agrument");invisible()},
+    interrupt=function(e) .burn(e)
+  )
 
-  block <- TRUE
-  if(exists('block',envir=env)) block <- get('block',envir=env)
-  if(block)
-    return(.getResponse())
+  pipeline <- FALSE
+  if(exists('pipeline',envir=env)) pipeline <- get('pipeline',envir=env)
+  if(!pipeline)
+    return(.getResponse(raw=raw))
   tryCatch(
     env$count <- env$count + 1,
     error = function(e) assign('count', 1, envir=env)
@@ -205,30 +168,92 @@ interrupt=function(e) .burn(e)
   invisible()
 }
 
-.redisRawCmd <- function(...)
+.renameCommand <- function(x, rep)
 {
-  con <- .redis()
-  f <- match.call()
-  n <- length(f) - 1
-  hdr <- paste('*', as.character(n), '\r\n',sep='')
-#  socketSelect(list(con),timeout=10L, write=TRUE)
-  cat(hdr, file=con)
-tryCatch({
-  for(j in seq_len(n)) {
-    v <- eval(f[[j+1]],envir=sys.frame(-1))
-    if(!is.raw(v)) v <- .cerealize(v)
-    l <- length(v)
-    hdr <- paste('$', as.character(l), '\r\n', sep='')
-#    socketSelect(list(con),timeout=10L, write=TRUE)
-    cat(hdr, file=con)
-#    socketSelect(list(con),timeout=10L, write=TRUE)
-    writeBin(v, con)
-#    socketSelect(list(con),timeout=10L, write=TRUE)
-    cat('\r\n', file=con)
-  }
-},
-error=function(e) {.redisError("Invalid agrument");invisible()},
-interrupt=function(e) .burn(e)
-)
-  .getResponse(raw=TRUE)
+  if(is.null(rep)) return(x)
+  v <- rawToChar(x)
+  if(v %in% names(rep)) return(charToRaw(rep[[v]]))
+  x
+}
+
+.getResponse <- function(raw=FALSE)
+{
+  env <- .redisEnv$current
+  tryCatch({
+    con <- .redis()
+    l <- readLines(con=con, n=1)
+
+    if(length(l)==0) .burn("Empty")
+    tryCatch(
+      env$count <- max(env$count - 1,0),
+      error = function(e) assign('count', 0, envir=env)
+    )
+    s <- substr(l, 1, 1)
+    if (nchar(l) < 2) {
+      if(s == "+") {
+        # '+' is a valid retrun message for at least one cmd (RANDOMKEY)
+        return("")
+      }
+      .burn("Invalid")
+    }
+    switch(s,
+         '-' = stop(substr(l,2,nchar(l))),
+         '+' = substr(l,2,nchar(l)),
+         ':' = as.numeric(substr(l,2,nchar(l))),
+         '$' = {
+             n <- as.numeric(substr(l,2,nchar(l)))
+             if (n < 0) {
+               return(NULL)
+             }
+             dat <- tryCatch(readBin(con, 'raw', n=n),
+                             error=function(e) .redisError(e$message))
+             m <- length(dat)
+             if(m==n)
+             {
+               l <- readLines(con,n=1)
+               if(raw)
+                 return(dat)
+               else
+                 return(tryCatch(unserialize(dat),
+                         error=function(e) rawToChar(dat)))
+             }
+# The message was not fully recieved in one pass for whatever reason.
+# We allocate a list to hold incremental messages and then concatenate it.
+# This perfromance enhancement was adapted from the Rbig server package, 
+# written by Steve Weston and Pat Shields.
+             rlen <- 50
+             j <- 1
+             r <- vector('list',rlen)
+             r[j] <- list(dat)
+             while(m<n)
+             {
+               dat <- tryCatch(readBin(con, 'raw', n=(n-m)),
+                            error=function (e) .redisError(e$message))
+               j <- j + 1
+               if(j>rlen)
+               {
+                 rlen <- 2*rlen
+                 length(r) <- rlen
+               }
+               r[j] <- list(dat)
+               m <- m + length(dat)
+             }
+             l <- readLines(con,n=1)  # Trailing \r\n
+             length(r) <- j
+             if(raw)
+               do.call(c,r)
+             else
+               tryCatch(unserialize(do.call(c,r)),
+                      error=function(e) rawToChar(do.call(c,r)))
+           },
+         '*' = {
+           numVars <- as.integer(substr(l,2,nchar(l)))
+           if(numVars > 0L) {
+             replicate(numVars, .getResponse(raw=raw), simplify=FALSE)
+           } else NULL
+         },
+       stop('Unknown message type')
+    )
+    }, interrupt=function(e) .burn(e)
+  )
 }
