@@ -1,4 +1,5 @@
-redisEval <- function(script, keys=vector("list",0), SHA=FALSE, ...) {
+redisEval <- function(script, keys=vector("list",0), SHA=FALSE, ...)
+{
   if(!is.list(keys)) keys = list(keys)
   numkeys = length(keys)
   if(numkeys>0) keys = as.character(keys)
