@@ -1,34 +1,34 @@
 # This file contains functions that operate on Redis 'string' values.
 
-redisGet <- function(key, raw=FALSE) {
-  if(raw) return(.redisRawCmd(.raw('GET'), .raw(key)))
-  .redisCmd(.raw('GET'), .raw(key))
+redisGet <- function(key, ...)
+{
+  .redisCmd(.raw('GET'), .raw(key), ...)
 }
 
-
-redisSet <- function(key, value, NX=FALSE) {
+redisSet <- function(key, value, NX=FALSE)
+{
   value <- .cerealize(value)
   cmd <- 'SET'
   if(NX) cmd <- 'SETNX'
   .redisCmd(.raw(cmd), .raw(key), value)
 }
 
-redisGetSet <- function(key, value) {
-  .redisCmd(.raw('GETSET'),.raw(key),value)
+redisGetSet <- function(key, value, ...)
+{
+  .redisCmd(.raw('GETSET'),.raw(key),value, ...)
 }
 
-redisMGet <- function(keys,raw=FALSE) {
+redisMGet <- function(keys, ...)
+{
   keylist <- as.list(keys)
-  if(raw)
-    x <- do.call('.redisRawCmd',lapply(c(list('MGET'),keylist),charToRaw))
-  else
-    x <- do.call('.redisCmd',lapply(c(list('MGET'),keylist),charToRaw))
+  x <- do.call('.redisCmd',c(lapply(c(list('MGET'),keylist),charToRaw),...))
 # The following may not occur, for example within a transaction block:
   if(length(x) == length(keylist)) names(x) <- keylist
   x
 }
 
-redisMSet <- function(keyvalues, NX=FALSE) {
+redisMSet <- function(keyvalues, NX=FALSE, ...)
+{
 # Includes a significant performance improvement contributed
 # by William Pleasant.
   if (0L == length(keyvalues))
@@ -38,6 +38,7 @@ redisMSet <- function(keyvalues, NX=FALSE) {
   j <- 2L * seq_along(keyvalues)
   a[j + 1L] <- keyvalues                # extends a
   a[j] <- lapply(names(keyvalues), charToRaw)
+  a <- c(a, ...)
   do.call('.redisCmd', a)
 }
 
@@ -51,6 +52,11 @@ redisIncrBy <- function(key, value)
   .redisCmd(.raw('INCRBY'),.raw(key),.raw(as.character(value)))
 }
 
+redisIncrByFloat <- function(key, value)
+{
+  .redisCmd(.raw('INCRBY'),.raw(key),.raw(as.character(value)))
+}
+
 redisDecrBy <- function(key, value)
 {
   .redisCmd(.raw('DECRBY'),.raw(key),.raw(as.character(value)))
